import { RestError } from 'byproxy-common';
async function post(url, data, includeCredentials) {
    const init = { method: 'POST', credentials: includeCredentials ? 'include' : 'same-origin', body: JSON.stringify(data) };
    const headers = new Headers();
    headers.append('Content-Type', 'application/json');
    init.headers = headers;
    const request = new Request(url, init);
    const response = await fetch(request);
    if (!response.ok) {
        const message = await response.text();
        throw new RestError(response.status, message);
    }
    return (await response.json());
}
function createRemote(url) {
    return function (message) {
        return new Promise((resolve, reject) => {
            post(url, message, false).then((response) => {
                if (response.error) {
                    reject(response.error);
                }
                else {
                    resolve(response.value);
                }
            }).catch((err) => {
                reject(err);
            });
        });
    };
}
function proxy(remote, path = []) {
    return new Proxy((function () { }), {
        get(_, prop, receiver) {
            if (prop === 'then') {
                if (path.length === 0) {
                    return { then: () => receiver };
                }
                const p = remote({ type: 'GET', path });
                return p.then.bind(p);
            }
            return proxy(remote, path.concat(prop));
        },
        set(_, prop, value) {
            return remote({ type: 'SET', path: path.concat(prop), args: [value] });
        },
        apply(_, __, argArray) {
            return remote({ type: 'APPLY', path, args: argArray || [] });
        }
    });
}
export function link(url) {
    return proxy(createRemote(url));
}
